#!/usr/bin/env ruby

## 2016-08-04

## usage: ruby ano-divt-topitem.rb [N = 500]

## - Overwrite M[*, 2] to fixed value ("1950/12/2").
## - For each master record, divide its transactions until #transactions >= N
##   while keeping the total amount of money.
## - Randomize T
##   - Invoices are randomly renamed
##   - Time is replaced by fixed value ("10:10")
##   - Some noise is added to unit price and quantity
## - Item IDs are replaced by the IDs included in the set of frequent item sets

def read_table(f = $stdin)
  n = f.gets.chomp.to_i
  (0...n).map{f.gets.chomp}
end

def read_table_skip(f = $stdin)
  n = f.gets.chomp.to_i
  n.times{f.gets}
end

def write_table(t, f = $stdout, no_line_num = false)
  f.puts t.size unless no_line_num
  f.puts t
end

def read_p(f = $stdin)
  read_table(f).map{|x| x.to_i - 1}
end

def write_p(p, f = $stdout, no_line_num = false)
  write_table(p.map{|x| x + 1}, f, no_line_num)
end

## res[i] = a[p[i]]
def unapply(a, p)
  p.map{|i| a[i]}
end

def shuffle(a)
  a.sort_by{rand}
end

def rand_perm(n)
  shuffle((0...n).to_a)
end



=begin
def ano_rename0(m, t)
  n = m.size
  y = rand_perm(n).map{|x| x.to_s}
  k2y = Hash.new
  m2 = []
  m.each do |s|
    a = s.split(/,/)
    k = a[0]
    k2y[k] = y[k2y.size] unless k2y[k]
    a[0] = k2y[k]
    m2 << a.join(',')
  end
  t2 = []
  t.each do |s|
    a = s.split(/,/)
    a[0] = k2y[a[0]]
    t2 << a.join(',')
  end
  return [m2, t2]
end
=end

def ano_rename(m, t)
  k = m.map{|s| s.split(/,/)[0]}.sort.uniq
  h = Hash[k.zip(shuffle(1..k.size))]
  [m, t].map{|x| x.map{|s| a = s.split(/,/); a[0] = h[a[0]]; a.join(",")}}
end

def ano_out(m0, t0, p)
  m, t = ano_rename(m0, t0)
  q = rand_perm(m.size)
  write_table(unapply(m, q))
  write_table(shuffle(t))
  write_p(unapply(p, q))
end

M_CID = 0
M_SEX = 1
M_BIRTHDAY = 2
M_COUNTRY = 3
M_SIZE = 4

T_CID = 0
T_INVOICE = 1
T_DATE = 2
T_TIME = 3
T_IID = 4
T_PRICE = 5
T_QUANTITY = 6
T_SIZE = 7

def max(x, y)
  x > y ? x : y
end

def min(x, y)
  x > y ? y : x
end

def abs(x)
  x < 0 ? -x : x
end

def t_split(ts, num = 1)
  return ts if ts.empty?
  st = []
  ts.each_with_index{|a, i| st << i if a[T_QUANTITY] > 1}
  num.times do
    break if st.empty?
    i = st.pop
    j = ts.size
    c = ts[i][T_QUANTITY]
    ts << ts[i].dup
    ts[i][T_QUANTITY] = c / 2
    ts[j][T_QUANTITY] = c - c / 2
    st << i
    st << j
  end
  return ts
end

## divide transactions upto r
def tr_div_upto(m, t, r = 100)
  h = Hash.new
  t.each do |x|
    a = x.split(/,/)
    key = a[T_CID]
    h[key] = [] unless h[key]
    a[T_QUANTITY] = a[T_QUANTITY].to_i
    h[key] << a
  end
  t2 = []
  h.each do |k, v|
    t_split(v, rand(10) + max(r - v.size, 0))
    v.each do |a|
      t2 << a.join(",")
    end
  end
  return t2
end

def read_int(dval)
  x = ARGV.shift
  x ? x.to_i : dval
end

def read_float(dval)
  x = ARGV.shift
  x ? x.to_f : dval
end

def read_array(a, dval)
  return dval unless n = a.shift
  (0...n.to_i).map{a.shift.to_i}
end




def rand_t(t)
  def rename_ary(a)
    h = Hash.new
    shuffle(a.sort.uniq).each{|k| h[k] = h.size + 1}
    a.map{|k| h[k]}
  end
  
  def rand_range(r0, r1)
    r0 + rand * (r1 - r0)
  end
  
  n = t.size
  t = t.map{|x| x.split(/,/)}.transpose
  t[T_INVOICE] = rename_ary(t[T_INVOICE])
  t[T_TIME] = (0...n).map{"10:10"}
  t[T_IID] = (0...n).map{"1234"}
  p = 0.05
  t[T_PRICE] = t[T_PRICE].map{|x| sprintf("%.2f", max(x.to_f * rand_range(1.0 - p, 1.0 + p), 0.01))}
  t[T_QUANTITY] = t[T_QUANTITY].map{|x| 1 + rand(x.to_i * 2 - 1)}
  return t.transpose.map{|a| a.join(",")}
end

def overwrite(m, ary)
  m.map do |x|
    a = x.split(/,/)
    ary.each_with_index{|v, i| a[i] = v if v}
    a.join(",")
  end
end

def item(t)
  x = [['23307'],['23388'],['22045'],['21121'],['21559'],['23298'],['22962'],['84997B'],['84988'],['47559B'],['22730'],['22628'],['23170'],['22993'],['20749'],['84991'],['21843'],['21914'],['22969'],['23293'],['23007'],['21578'],['21213'],['22692'],['23173'],['22635'],['22722'],['47566'],['23077'],['22374'],['22716'],['21242'],['23175'],['20728'],['21210'],['20712'],['84997C'],['22666'],['23008'],['23243'],['23295'],['22077'],['20685'],['22712'],['21217'],['22627'],['22634'],['22557'],['23308'],['23294'],['22139'],['23174'],['21981'],['21770'],['22960'],['37495'],['22846'],['22907'],['85014A'],['22626'],['23319'],['23199'],['23291'],['21156'],['84375'],['22693'],['21481'],['22192'],['22847'],['23346'],['22333'],['21915'],['22620'],['85014B'],['21936'],['22384'],['23238'],['22915'],['23241'],['23292'],['23389'],['21977'],['22659'],['23204'],['21791'],['22027'],['22908'],['22619'],['23206'],['22090'],['23209'],['20724', '22326'],['21975', 'POST'],['20719', '22326'],['22138', '22617'],['21080', 'POST'],['22417', '21976'],['22423', '22727'],['22556', '22631'],['22326', '20725'],['POST', '23236'],['POST', '20725'],['23240', '23236'],['22556', '22630'],['22423', '23245'],['21080', '22326'],['POST', '23203'],['21731', '22728'],['22725', '22728'],['22423', '21731'],['22423', '21232'],['POST', '21212'],['22423', '23108'],['23245', '23236'],['POST', '20724'],['20676', '20677'],['22699', '22697'],['22417', 'POST'],['22382', '22326'],['POST', '22544'],['22625', '22624'],['POST', '22625'],['22467', 'POST'],['POST', '21232'],['20675', '20676'],['22382', '20726'],['POST', '22029'],['POST', '84997D'],['21731', '23084'],['POST', '23480'],['POST', '22951'],['21080', '21212'],['POST', '85099B'],['20726', '20725'],['22629', '22555'],['POST', '22728'],['POST', '22492'],['22329', 'POST'],['22138', '22326'],['22629', '22554'],['47590B', '47590A'],['22539', '22544'],['22423', '22699'],['POST', '23108'],['POST', '23198'],['23245', '23240'],['22630', '22554'],['POST', '22356'],['22423', '22630'],['POST', '22367'],['POST', '20719'],['POST', '22961'],['POST', '23084'],['22423', '22728'],['22726', '22729'],['POST', '22352'],['85099B', '22326'],['23245', '22720'],['22138', 'POST'],['23159', 'POST'],['22417', '21212'],['22629', '22556'],['22728', '22729'],['23084', '23480'],['22630', '22555'],['23245', 'POST'],['22698', '22699'],['POST', '23240'],['22329', '22326'],['22629', '23256'],['POST', '22624'],['POST', '22555', '22326'],['POST', '22554', '22555'],['POST', '22555', '22551'],['POST', '22727', '22726'],['22423', 'POST', '22326'],['POST', '20750', '22326'],['22726', '22727', '22728'],['POST', '22551', '22326'],['22556', 'POST', '22326'],['22629', '22630', '22551'],['21080', '21094', '21086'],['22629', '22630', '22631'],['22556', '22554', '22326'],['22556', '22555', '22551'],['POST', '23256', '23254'],['POST', '22554', '22551'],['22554', '22326', '22551'],['22725', '22726', '22727'],['22629', '22631', '22326'],['POST', '22554', '22326'],['22629', 'POST', '22631'],['22556', 'POST', '22554'],['22556', '22555', '22554'],['POST', '21731', '22326'],['22556', '22554', '22551'],['POST', '20726', '22326'],['POST', '21094', '21086'],['23256', '23255', '23254'],['22423', 'POST', '22554'],['22423', 'POST', '22720'],['22556', 'POST', '22551'],['22328', '22630', 'POST', '22326'],['22328', '22629', 'POST', '22326'],['22630', '22629', 'POST', '22326']].flatten
  n = x.size
  t.each_with_index do |ti,i|
    a = ti.split(/,/)
    a[4] = x[i % n]
    t[i] = a.join(",")
  end
end

## input (M, T)
m = read_table()
t = read_table()

## body
r = read_int(50)
n = m.size
p = (0...n).to_a
m = overwrite(m, [nil, nil, "1950/12/2"])
t = rand_t(t)
t = tr_div_upto(m, t, r)
t = item(t)

## output (P)
ano_out(m, t, p)
