#!/usr/bin/env ruby

## 2016-08-04

## usage: ruby ano-divt.rb [N = 500]

## - Overwrite M[*, 2] to fixed value ("1950/12/2").
## - For each master record, divide its transactions until #transactions >= N
##   while keeping the total amount of money.
## - Randomize T
##   - Invoices are randomly renamed
##   - Time is replaced by fixed value ("10:10")
##   - Item ID is replaced by fixed value ("1234")
##   - Some noise is added to unit price and quantity

def read_table(f = $stdin)
  n = f.gets.chomp.to_i
  (0...n).map{f.gets.chomp}
end

def read_table_skip(f = $stdin)
  n = f.gets.chomp.to_i
  n.times{f.gets}
end

def write_table(t, f = $stdout, no_line_num = false)
  f.puts t.size unless no_line_num
  f.puts t
end

def read_p(f = $stdin)
  read_table(f).map{|x| x.to_i - 1}
end

def write_p(p, f = $stdout, no_line_num = false)
  write_table(p.map{|x| x + 1}, f, no_line_num)
end


## res[i] = a[p[i]]
def unapply(a, p)
  p.map{|i| a[i]}
end

def shuffle(a)
  a.sort_by{rand}
end

def rand_perm(n)
  shuffle((0...n).to_a)
end

=begin
def ano_rename0(m, t)
  n = m.size
  y = rand_perm(n).map{|x| x.to_s}
  k2y = Hash.new
  m2 = []
  m.each do |s|
    a = s.split(/,/)
    k = a[0]
    k2y[k] = y[k2y.size] unless k2y[k]
    a[0] = k2y[k]
    m2 << a.join(',')
  end
  t2 = []
  t.each do |s|
    a = s.split(/,/)
    a[0] = k2y[a[0]]
    t2 << a.join(',')
  end
  return [m2, t2]
end
=end

def ano_rename(m, t)
  k = m.map{|s| s.split(/,/)[0]}.sort.uniq
  h = Hash[k.zip(shuffle(1..k.size))]
  [m, t].map{|x| x.map{|s| a = s.split(/,/); a[0] = h[a[0]]; a.join(",")}}
end

def ano_out(m0, t0, p)
  m, t = ano_rename(m0, t0)
  q = rand_perm(m.size)
  write_table(unapply(m, q))
  write_table(shuffle(t))
  write_p(unapply(p, q))
end

M_CID = 0
M_SEX = 1
M_BIRTHDAY = 2
M_COUNTRY = 3
M_SIZE = 4

T_CID = 0
T_INVOICE = 1
T_DATE = 2
T_TIME = 3
T_IID = 4
T_PRICE = 5
T_QUANTITY = 6
T_SIZE = 7

def max(x, y)
  x > y ? x : y
end

def min(x, y)
  x > y ? y : x
end

def abs(x)
  x < 0 ? -x : x
end

def t_split(ts, num = 1)
  return ts if ts.empty?
  st = []
  ts.each_with_index{|a, i| st << i if a[T_QUANTITY] > 1}
  num.times do
    break if st.empty?
    i = st.pop
    j = ts.size
    c = ts[i][T_QUANTITY]
    ts << ts[i].dup
    ts[i][T_QUANTITY] = c / 2
    ts[j][T_QUANTITY] = c - c / 2
    st << i
    st << j
  end
  return ts
end

## divide transactions upto r
def tr_div_upto(m, t, r = 100)
  h = Hash.new
  t.each do |x|
    a = x.split(/,/)
    key = a[T_CID]
    h[key] = [] unless h[key]
    a[T_QUANTITY] = a[T_QUANTITY].to_i
    h[key] << a
  end
  t2 = []
  h.each do |k, v|
    t_split(v, rand(10) + max(r - v.size, 0))
    v.each do |a|
      t2 << a.join(",")
    end
  end
  return t2
end

def read_int(dval)
  x = ARGV.shift
  x ? x.to_i : dval
end

def read_float(dval)
  x = ARGV.shift
  x ? x.to_f : dval
end

def read_array(a, dval)
  return dval unless n = a.shift
  (0...n.to_i).map{a.shift.to_i}
end




def rand_t(t)
  def rename_ary(a)
    h = Hash.new
    shuffle(a.sort.uniq).each{|k| h[k] = h.size + 1}
    a.map{|k| h[k]}
  end
  
  def rand_range(r0, r1)
    r0 + rand * (r1 - r0)
  end
  
  n = t.size
  t = t.map{|x| x.split(/,/)}.transpose
  t[T_INVOICE] = rename_ary(t[T_INVOICE])
  t[T_TIME] = (0...n).map{"10:10"}
  t[T_IID] = (0...n).map{"1234"}
  p = 0.05
  t[T_PRICE] = t[T_PRICE].map{|x| sprintf("%.2f", max(x.to_f * rand_range(1.0 - p, 1.0 + p), 0.01))}
  t[T_QUANTITY] = t[T_QUANTITY].map{|x| 1 + rand(x.to_i * 2 - 1)}
  return t.transpose.map{|a| a.join(",")}
end

def overwrite(m, ary)
  m.map do |x|
    a = x.split(/,/)
    ary.each_with_index{|v, i| a[i] = v if v}
    a.join(",")
  end
end

## input (M, T)
m = read_table()
t = read_table()

## body
r = read_int(500)
n = m.size
p = (0...n).to_a
m = overwrite(m, [nil, nil, "1950/12/2"])
t = rand_t(t)
t = tr_div_upto(m, t, r)

## output (P)
ano_out(m, t, p)
