#!/usr/bin/env ruby

## 2016-08-24

## - Mean absolute error of two corss tabulations
##   (1) cross(M, T) vs cross(M', T')
##   (2) cross(M, T) vs corss(M, P, T')
## - Cross table is separated by sex(M[,1]) and country(M[,3])
## - The aggregate function is "mean unit price", i.e. (total amount of money) / (total number of items)

M_CID = 0
M_SEX = 1
M_BIRTHDAY = 2
M_COUNTRY = 3
M_SIZE = 4

T_CID = 0
T_INVOICE = 1
T_DATE = 2
T_TIME = 3
T_IID = 4
T_PRICE = 5
T_QUANTITY = 6
T_SIZE = 7

def read_table(f = $stdin)
  n = f.gets.chomp.to_i
  (0...n).map{f.gets.chomp}
end

def read_table_skip(f = $stdin)
  n = f.gets.chomp.to_i
  n.times{f.gets}
end

def write_table(t, f = $stdout, no_line_num = false)
  f.puts t.size unless no_line_num
  f.puts t
end

def read_p(f = $stdin)
  read_table(f).map{|x| x.to_i - 1}
end

def write_p(p, f = $stdout, no_line_num = false)
  write_table(p.map{|x| x + 1}, f, no_line_num)
end

## res[i] = a[p[i]]
def unapply(a, p)
  p.map{|i| a[i]}
end

def shuffle(a)
  a.sort_by{rand}
end

def rand_perm(n)
  shuffle((0...n).to_a)
end

def sum(a)
  a.inject(0){|x, y| x + y}
end

def mean(a)
  return 0 if a.empty?
  return sum(a).to_f / a.size
end

def abs(x)
  x < 0 ? -x : x
end

def mae(a, b, value_for_half_missing = 1e10)
  n = a.size
  mean((0...n).map{|i| (a[i] == b[i]) ? 0 : (a[i]!=nil && b[i]!=nil ? abs(a[i] - b[i]) : value_for_half_missing)})
end


def transactions(m, t, b = 0, e = -1)
  n = m.size
  mkeys = m.map{|x| x.split(/,/)[M_CID]}
  h = Hash[mkeys.zip((0...n).to_a)]
  h.default = nil
  r = Array.new(n){[]}
  t.each do |x|
    a = x.split(/,/)
    k = a[T_CID]
    r[h[k]] << a[b..e] if h[k]
  end
  return r.map{|x| x.sort}
end

def read_int(dval)
  x = ARGV.shift
  x ? x.to_i : dval
end

def read_float(dval)
  x = ARGV.shift
  x ? x.to_f : dval
end

def read_array(a, dval)
  return dval unless n = a.shift
  (0...n.to_i).map{a.shift.to_i}
end

def to_key(s, key_ids)
  x = s.split(/,/)
  key_ids.map{|i| x[i]}.join(",")
end

def cross(m, t, key_ids)
  n = m.size
  x = transactions(m, t)
  keys = m.map{|s| to_key(s, key_ids)}
  h = Hash.new(nil)
  keys.each_with_index do |k, i|
    h[k] = [] unless h[k]
    h[k] += x[i]
  end
  return h
end

def cross_mae(h1, h2)
  return mean(h1.to_a.map{|k, v| abs(yield(h1[k]) - yield(h2[k]))})
end

def composit(m1, m2, w)
  n = m1.size
  m = w.size
  (0...n).map do |i|
    x1 = m1[i].split(/,/)
    x2 = m2[i].split(/,/)
    (0...m).map{|i| w[i] ? x1[i] : x2[i]}.join(",")
  end
end

def agg_avg_unitprice(ts)
  return 0.0 unless ts
  total = 0.0
  ct = 0
  ts.each do |x|
    u = x[T_PRICE].to_f
    c = x[T_QUANTITY].to_i
    total += u * c
    ct += c
  end
  return ct == 0 ? 0 : total / ct
end

## input (M, T, M', T', P)
m1 = read_table()
t1 = read_table()
m2 = read_table()
t2 = read_table()
p = read_p()

## body
key_ids = read_array(ARGV, [1,3])
h1 = cross(m1, t1, key_ids) # M, T
h2 = cross(m2, t2, key_ids) # M', T'
m3 = composit(unapply(m1, p), m2, [false, true, true, true])
h3 = cross(m3, t2, key_ids) # M, f, T'
e1 = cross_mae(h1, h2){|x| agg_avg_unitprice(x)} # MT vs M'T'
e2 = cross_mae(h1, h3){|x| agg_avg_unitprice(x)} # MT vs MfT'

## output [MT vs M'T', MT vs MfT']
write_table([e1, e2])
